--[[
	Project: LibCoroutine
	Version: 1.0
	Author: Zoc ( http://www.riftui.com/forums/member.php?u=1215 )
	
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
]]

if not Library then Library = {} end
if not Library.LibCoroutine then Library.LibCoroutine = {} end

-- 2^16 coroutines should be enough, and should avoid collisions.
local maxCoroutines = 65536

-- This MUST start at 1, starting with 0 does not work.
local lastCoroutine = 1

-- Ugly function, but the collisions shouldn't happen a lot.
local function ValidID( id )
	for i,v in ipairs(Event.System.Update.Begin) do
		if ( v[4] == id ) then
			-- Debug ( Use this if you suspect of collisions happening all the time.)
			-- print( "Coroutine Collision Happened, ID: " .. id )
			return false
		end
	end
	return true
end

local function IncrementLastCoroutineID()
	lastCoroutine = lastCoroutine + 1
	if ( lastCoroutine >= maxCoroutines ) then
	-- I wish 0 worked here, so I could use the handy % operator, instead of a whole function :(
		lastCoroutine = 1
	end
end

-- returns coroutine ID. Also, send id as parameter to the coroutine (so it can kill itself later). Any other parameters get sent as well, after the id.
function Library.LibCoroutine.RegisterCoroutine( addonName, process, ... )
	while ( not( ValidID( lastCoroutine ) ) ) do
		IncrementLastCoroutineID()
	end
	local assignedID = lastCoroutine
	
--	print("[addon = " .. addonName .. "] Coroutine: " .. tostring( process ) .. " received ID: " .. assignedID )
	IncrementLastCoroutineID()

	local coro = nil
	if ( ... == nil ) then
		coro = coroutine.wrap( function () process( assignedID ) end )
	else
		local arg = {...}
		coro = coroutine.wrap( function () process( assignedID, unpack(arg) ) end )
	end

	table.insert( Event.System.Update.Begin, {coro, addonName, "OnUpdate", assignedID } )

	return assignedID
end

function Library.LibCoroutine.RemoveCoroutine( id )
	for i,v in ipairs( Event.System.Update.Begin ) do
		if ( v[4] == id ) then
--			print ( "Found and killed id #" .. id )
			table.remove( Event.System.Update.Begin, i )	
		end
	end
end 

function Library.LibCoroutine.Pause( seconds )
	local start = Inspect.Time.Frame()
	while start + seconds > Inspect.Time.Frame() do
		coroutine.yield()
	end
end
